﻿//----------------------------------------------------------------------------
// <copyright file="DataAccessProfile.cs" company="Russell Wilkins">
//     Copyright (c) Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//----------------------------------------------------------------------------
namespace Dax
{
    using System.ComponentModel;
    using System.IO;

    /// <summary>
    /// Class to hold the properties used for the Data Access generation.
    /// </summary>
    public class DataAccessProfile : INotifyPropertyChanged
    {
        private bool isDirty = false;
        private string generationNamespace;
        private string testNamespace;
        private string concurrencyColumn;
        private string outputPath;
        private string testOutputPath;
        private string recordsFolder;
        private string buildersFolder;
        private string repositoriesFolder;
        private string projectFile;
        private string testVSProjectFile;
        private bool clearOutputDirectories;

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the DataAccessProfile class.
        /// </summary>
        public DataAccessProfile()
        {
        }
        #endregion

        /// <summary>
        /// INotifyPropertyChanged event.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #region Public properties
        /// <summary>
        /// Gets or sets the Namespace.
        /// </summary>
        public string Namespace
        {
            get
            {
                return this.generationNamespace;
            }

            set
            {
                if (this.generationNamespace != value)
                {
                    this.generationNamespace = value;
                    this.OnPropertyChanged("Namespace");
                }
            }
        }

        /// <summary>
        /// Gets or sets the TestNamespace.
        /// </summary>
        public string TestNamespace
        {
            get
            {
                return this.testNamespace;
            }

            set
            {
                if (this.testNamespace != value)
                {
                    this.testNamespace = value;
                    this.OnPropertyChanged("TestNamespace");
                }
            }
        }

        /// <summary>
        /// Gets or sets the ConcurrencyColumn name.
        /// </summary>
        public string ConcurrencyColumn
        {
            get
            {
                return this.concurrencyColumn;
            }

            set
            {
                if (this.concurrencyColumn != value)
                {
                    this.concurrencyColumn = value;
                    this.OnPropertyChanged("ConcurrencyColumn");
                }
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the output directories should be cleared before generation.
        /// </summary>
        public bool ClearOutputDirectories
        {
            get
            {
                return this.clearOutputDirectories;
            }

            set
            {
                if (this.clearOutputDirectories != value)
                {
                    this.clearOutputDirectories = value;
                    this.OnPropertyChanged("ClearOutputDirectories");
                }
            }
        }

        /// <summary>
        /// Gets or sets the base output path for generated files.
        /// </summary>
        public string OutputPath
        {
            get
            {
                return this.outputPath;
            }

            set
            {
                if (this.outputPath != value)
                {
                    this.outputPath = value;
                    this.OnPropertyChanged("OutputPath");
                }
            }
        }

        /// <summary>
        /// Gets or sets the base output path for generated test files.
        /// </summary>
        public string TestOutputPath
        {
            get
            {
                return this.testOutputPath;
            }

            set
            {
                if (this.testOutputPath != value)
                {
                    this.testOutputPath = value;
                    this.OnPropertyChanged("TestOutputPath");
                }
            }
        }

        /// <summary>
        /// Gets or sets the folder for the generated DataRecord class files.
        /// </summary>
        public string RecordsFolder
        {
            get
            {
                return this.recordsFolder;
            }

            set
            {
                if (this.recordsFolder != value)
                {
                    this.recordsFolder = value;
                    this.OnPropertyChanged("RecordsFolder");
                }
            }
        }

        /// <summary>
        /// Gets or sets the folder for the generated DataRecordBuilder class files.
        /// </summary>
        public string BuildersFolder
        {
            get
            {
                return this.buildersFolder;
            }

            set
            {
                if (this.buildersFolder != value)
                {
                    this.buildersFolder = value;
                    this.OnPropertyChanged("BuildersFolder");
                }
            }
        }

        /// <summary>
        /// Gets or sets the folder for the generated DataAccessRepository class files.
        /// </summary>
        public string RepositoriesFolder
        {
            get
            {
                return this.repositoriesFolder;
            }

            set
            {
                if (this.repositoriesFolder != value)
                {
                    this.repositoriesFolder = value;
                    this.OnPropertyChanged("RepositoriesFolder");
                }
            }
        }

        /// <summary>
        /// Gets or sets the Visual Studio project file.
        /// </summary>
        public string VSProjectFile
        {
            get
            {
                return this.projectFile;
            }

            set
            {
                if (this.projectFile != value)
                {
                    this.projectFile = value;
                    this.OnPropertyChanged("VSProjectFile");
                }
            }
        }

        /// <summary>
        /// Gets or sets the Visual Studio project file.
        /// </summary>
        public string TestVSProjectFile
        {
            get
            {
                return this.testVSProjectFile;
            }

            set
            {
                if (this.testVSProjectFile != value)
                {
                    this.testVSProjectFile = value;
                    this.OnPropertyChanged("TestVSProjectFile");
                }
            }
        }

        /// <summary>
        /// Gets the full path for the generated DataRecord class files.
        /// </summary>
        public string RecordsFolderFullPath
        {
            get
            {
                return Path.Combine(this.OutputPath, this.RecordsFolder);
            }
        }

        /// <summary>
        /// Gets the full path for the generated DataRecordBuilder class files.
        /// </summary>
        public string BuildersFolderFullPath
        {
            get
            {
                return Path.Combine(this.OutputPath, this.BuildersFolder);
            }
        }

        /// <summary>
        /// Gets the full path for the generated DataRepository class files.
        /// </summary>
        public string RepositoriesFolderFullPath
        {
            get
            {
                return Path.Combine(this.OutputPath, this.RepositoriesFolder);
            }
        }

        /// <summary>
        /// Gets the full path for the generated DataRepository test class files.
        /// </summary>
        public string TestRecordsFolderFullPath
        {
            get
            {
                return Path.Combine(this.TestOutputPath, this.RecordsFolder);
            }
        }

        /// <summary>
        /// Gets the full path for the generated DataRepository test class files.
        /// </summary>
        public string TestBuildersFolderFullPath
        {
            get
            {
                return Path.Combine(this.TestOutputPath, this.BuildersFolder);
            }
        }

        /// <summary>
        /// Gets the full path for the generated DataRepository test class files.
        /// </summary>
        public string TestRepositoriesFolderFullPath
        {
            get
            {
                return Path.Combine(this.TestOutputPath, this.RepositoriesFolder);
            }
        }
        #endregion

        internal bool IsDirty
        {
            get
            {
                return this.isDirty;
            }

            set
            {
                this.isDirty = value;
            }
        }

        /// <summary>
        /// Gets the full path for a generated DataRecord class file.
        /// </summary>
        /// <param name="fileName">The generated file name.</param>
        /// <returns>The full file path.</returns>
        public string RecordFilePath(string fileName)
        {
            return Path.Combine(this.RecordsFolderFullPath, fileName);
        }

        /// <summary>
        /// Gets the full path for a generated DataRecord class file.
        /// </summary>
        /// <param name="fileName">The generated file name.</param>
        /// <returns>The full file path.</returns>
        public string TestRecordFilePath(string fileName)
        {
            return Path.Combine(this.TestRecordsFolderFullPath, fileName);
        }

        /// <summary>
        /// Gets the full path for a generated DataRecordBuilder class file.
        /// </summary>
        /// <param name="fileName">The generated file name.</param>
        /// <returns>The full file path.</returns>
        public string BuilderFilePath(string fileName)
        {
            return Path.Combine(this.BuildersFolderFullPath, fileName);
        }

        /// <summary>
        /// Gets the full path for a generated DataRecordBuilder class file.
        /// </summary>
        /// <param name="fileName">The generated file name.</param>
        /// <returns>The full file path.</returns>
        public string TestBuilderFilePath(string fileName)
        {
            return Path.Combine(this.TestBuildersFolderFullPath, fileName);
        }

        /// <summary>
        /// Gets the full path for a generated DataRepository class file.
        /// </summary>
        /// <param name="fileName">The generated file name.</param>
        /// <returns>The full file path.</returns>
        public string RepositoryFilePath(string fileName)
        {
            return Path.Combine(this.RepositoriesFolderFullPath, fileName);
        }

        /// <summary>
        /// Gets the full path for a generated DataRepository class file.
        /// </summary>
        /// <param name="fileName">The generated file name.</param>
        /// <returns>The full file path.</returns>
        public string TestRepositoryFilePath(string fileName)
        {
            return Path.Combine(this.TestRepositoriesFolderFullPath, fileName);
        }

        /// <summary>
        /// Gets the full path for a generated file in the base output path.
        /// </summary>
        /// <param name="fileName">The generated file name.</param>
        /// <returns>The full file path.</returns>
        public string TestOutputFilePath(string fileName)
        {
            return Path.Combine(this.TestOutputPath, fileName);
        }

        /// <summary>
        /// Gets the full path for a generated file in the base output path.
        /// </summary>
        /// <param name="fileName">The generated file name.</param>
        /// <returns>The full file path.</returns>
        public string OutputFilePath(string fileName)
        {
            return Path.Combine(this.OutputPath, fileName);
        }

        private void OnPropertyChanged(string name)
        {
            this.isDirty = true;

            PropertyChangedEventHandler handler = this.PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }
    }
}
