﻿//-----------------------------------------------------------------------------
// <copyright file="GenerationProfile.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//----------------------------------------------------------------------------
namespace Dax
{
    using System.Collections.ObjectModel;
    using System.ComponentModel;

    /// <summary>
    /// Class to hold the properties used for the generation.
    /// </summary>
    public class GenerationProfile : INotifyPropertyChanged
    {
        private bool isDirty = false;
        private string connectionString;
        private bool trace;
        private PluralType pluralizer;

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the GenerationProfile class.
        /// </summary>
        public GenerationProfile()
        {
            this.ORM = new ORMProfile();
            this.DataAccess = new DataAccessProfile();
            this.DataAccessInterface = new DataAccessInterfaceProfile();
            this.StoredProcedures = new StoredProcedureProfile();
            this.TableExclusions = new ObservableCollection<string>();

            this.TableExclusions.CollectionChanged += new System.Collections.Specialized.NotifyCollectionChangedEventHandler(this.TableExclusions_CollectionChanged);
        }
        #endregion

        /// <summary>
        /// INotifyPropertyChanged event.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #region Public properties

        /// <summary>
        /// Gets or sets a value indicating whether this objects properties have changed.
        /// </summary>
        public bool IsDirty
        {
            get
            {
                return this.isDirty
                    || this.DataAccess.IsDirty
                    || this.DataAccessInterface.IsDirty
                    || this.StoredProcedures.IsDirty
                    || this.ORM.IsDirty;
            }

            set
            {
                this.isDirty = value;
                this.ORM.IsDirty = value;
                this.DataAccessInterface.IsDirty = value;
                this.DataAccess.IsDirty = value;
                this.StoredProcedures.IsDirty = value;
                this.ORM.IsDirty = value;
            }
        }
        
        /// <summary>
        /// Gets or sets the ConnectionString.
        /// </summary>
        /// <value>The ConnectionString.</value>
        public string ConnectionString
        {
            get
            {
                return this.connectionString;
            }

            set
            {
                if (this.connectionString != value)
                {
                    this.connectionString = value;
                    this.OnPropertyChanged("ConnectionString");
                }
            }
        }

        /// <summary>
        /// Gets or sets the DataAccess generation profile.
        /// </summary>
        public DataAccessProfile DataAccess
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the DataAccessInterface generation profile.
        /// </summary>
        public DataAccessInterfaceProfile DataAccessInterface
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the StoredProcedure generation profile.
        /// </summary>
        public StoredProcedureProfile StoredProcedures
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the ORM generation profile.
        /// </summary>
        public ORMProfile ORM
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the pluralization option.
        /// </summary>
        public PluralType Pluralizer
        {
            get
            {
                return this.pluralizer;
            }

            set
            {
                if (this.pluralizer != value)
                {
                    this.pluralizer = value;
                    this.OnPropertyChanged("Pluralizer");
                }
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether trace statement are generated.
        /// </summary>
        public bool Trace
        {
            get
            {
                return this.trace;
            }

            set
            {
                if (this.trace != value)
                {
                    this.trace = value;
                    this.OnPropertyChanged("Trace");
                }
            }
        }

        /// <summary>
        /// Gets or sets a collection of table names to exclude from generation.
        /// </summary>
        public ObservableCollection<string> TableExclusions
        {
            get;
            set;
        }
        #endregion

        private void OnPropertyChanged(string name)
        {
            this.isDirty = true;

            PropertyChangedEventHandler handler = this.PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }

        private void TableExclusions_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            this.isDirty = true;
        }
    }
}
