﻿//----------------------------------------------------------------------------
// <copyright file="ORMProfile.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//----------------------------------------------------------------------------
namespace Dax
{
    using System.ComponentModel;
    using System.IO;

    /// <summary>
    /// Class to hold the properties used for the ORM generation.
    /// </summary>
    public class ORMProfile : INotifyPropertyChanged
    {
        private bool isDirty = false;
        private string generationNamespace;
        private bool enabled;
        private string outputPath;
        private string objectsFolder;
        private string repositoriesFolder;
        private string projectFile;

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the ORMProfile class.
        /// </summary>
        public ORMProfile()
        {
        }
        #endregion

        /// <summary>
        /// INotifyPropertyChanged event.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #region Public properties
        /// <summary>
        /// Gets or sets the Namespace.
        /// </summary>
        /// <value>The Namespace.</value>
        public string Namespace
        {
            get
            {
                return this.generationNamespace;
            }

            set
            {
                if (this.generationNamespace != value)
                {
                    this.generationNamespace = value;
                    this.OnPropertyChanged("Namespace");
                }
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether ORM generation is enabled.
        /// </summary>
        public bool Enabled
        {
            get
            {
                return this.enabled;
            }

            set
            {
                if (this.enabled != value)
                {
                    this.enabled = value;
                    this.OnPropertyChanged("Enabled");
                }
            }
        }

        /// <summary>
        /// Gets or sets the base output path for generated files.
        /// </summary>
        public string OutputPath
        {
            get
            {
                return this.outputPath;
            }

            set
            {
                if (this.outputPath != value)
                {
                    this.outputPath = value;
                    this.OnPropertyChanged("OutputPath");
                }
            }
        }

        /// <summary>
        /// Gets or sets the folder for the generated relational class files.
        /// </summary>
        public string ObjectsFolder
        {
            get
            {
                return this.objectsFolder;
            }

            set
            {
                if (this.objectsFolder != value)
                {
                    this.objectsFolder = value;
                    this.OnPropertyChanged("ObjectsFolder");
                }
            }
        }

        /// <summary>
        /// Gets or sets the folder for the generated repository class files.
        /// </summary>
        public string RepositoriesFolder
        {
            get
            {
                return this.repositoriesFolder;
            }

            set
            {
                if (this.repositoriesFolder != value)
                {
                    this.repositoriesFolder = value;
                    this.OnPropertyChanged("RepositoriesFolder");
                }
            }
        }

        /// <summary>
        /// Gets or sets the Visual Studion project file.
        /// </summary>
        public string VSProjectFile
        {
            get
            {
                return this.projectFile;
            }

            set
            {
                if (this.projectFile != value)
                {
                    this.projectFile = value;
                    this.OnPropertyChanged("VSProjectFile");
                }
            }
        }

        /// <summary>
        /// Gets the full path for the generated relational class files.
        /// </summary>
        public string ObjectsFolderFullPath
        {
            get
            {
                return Path.Combine(this.OutputPath, this.ObjectsFolder);
            }
        }

        /// <summary>
        /// Gets the full path for the generated repository class files.
        /// </summary>
        public string RepositoriesFolderFullPath
        {
            get
            {
                return Path.Combine(this.OutputPath, this.RepositoriesFolder);
            }
        }
        #endregion

        internal bool IsDirty
        {
            get
            {
                return this.isDirty;
            }

            set
            {
                this.isDirty = value;
            }
        }

        /// <summary>
        /// Gets the full path for a generated file in the objects output path.
        /// </summary>
        /// <param name="fileName">The generated file name.</param>
        /// <returns>The full file path.</returns>
        public string ObjectsFilePath(string fileName)
        {
            return Path.Combine(this.ObjectsFolderFullPath, fileName);
        }

        /// <summary>
        /// Gets the full path for a generated file in the base output path.
        /// </summary>
        /// <param name="fileName">The generated file name.</param>
        /// <returns>The full file path.</returns>
        public string OutputFilePath(string fileName)
        {
            return Path.Combine(this.OutputPath, fileName);
        }

        /// <summary>
        /// Gets the full path for a generated file in the repositories output path.
        /// </summary>
        /// <param name="fileName">The generated file name.</param>
        /// <returns>The full file path.</returns>
        public string RepositoriesFilePath(string fileName)
        {
            return Path.Combine(this.RepositoriesFolderFullPath, fileName);
        }

        private void OnPropertyChanged(string name)
        {
            this.isDirty = true;

            PropertyChangedEventHandler handler = this.PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(name));
            }
        }
    }
}
