﻿//-----------------------------------------------------------------------
// <copyright file="SqlServerDatabaseBuilder.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax.Database
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Data.SqlClient;
    using System.Diagnostics;
    using System.Data;

    /// <summary>
    /// TODO: Describe the class
    /// </summary>
    public class SqlServerDBBuilder
    {
        private string connectionString;
        private int schemaCount;
        private int tableCount;
        private int typeCount;
        private int columnCount;
        private int primaryKeyCount;
        private int relationshipCount;
        private int relationshipColumnCount;
        private int procedureCount;
        private int parameterCount;
        private SqlConnection connection;

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the SqlServerDatabaseBuilder class.
        /// </summary>
        public SqlServerDBBuilder(string connectionString)
        {
            this.connectionString = connectionString;
        }
        #endregion

        public DBDatabase Get()
        {
            Trace.TraceInformation("------ Loading database information: {0} ------", connectionString);

            DBDatabase dbDatabase;

            using (connection = new SqlConnection(this.connectionString))
            {
                connection.Open();

                dbDatabase = new DBDatabase(connection.Database);

                this.LoadSchemas(dbDatabase);
                this.LoadTypes(dbDatabase);
                this.LoadTables(dbDatabase);
                this.LoadTableColumns(dbDatabase);
                this.LoadPrimaryKeys(dbDatabase);
                this.LoadRelationships(dbDatabase);
                this.LoadRelationshipColumns(dbDatabase);
                this.LoadProcedures(dbDatabase);
                this.LoadParameters(dbDatabase);

                connection.Close();
            }

            Trace.TraceInformation("========== Database information complete ==========");
            Trace.TraceInformation(this.schemaCount + " schemas");
            Trace.TraceInformation(this.typeCount + " types");
            Trace.TraceInformation(this.tableCount + " tables");
            Trace.TraceInformation(this.primaryKeyCount + " primary keys");
            Trace.TraceInformation(this.columnCount + " columns");
            Trace.TraceInformation(this.relationshipCount + " relationships");
            Trace.TraceInformation(this.relationshipColumnCount + " relationship columns");
            Trace.TraceInformation(this.procedureCount + " procedures");
            Trace.TraceInformation(this.parameterCount + " parameters");
            Trace.TraceInformation("========== Database information complete ==========");

            return dbDatabase;
        }


        private void LoadSchemas(DBDatabase dbDatabase)
        {
            Trace.TraceInformation("------ Loading schemas ------");

            SqlDataReader reader = this.ExecuteCommand("select * from sys.schemas");
            while (reader.Read())
            {
                DBSchema dbObject = new DBSchema(dbDatabase, (int)reader["schema_id"], reader["name"] as string);
                dbDatabase.Schemas.Add(dbObject.Id, dbObject);
                this.schemaCount++;

                TraceObject(dbObject);
            }

            reader.Close();
        }

        private void LoadTypes(DBDatabase dbDatabase)
        {
            Trace.TraceInformation("------ Loading types ------");

            SqlDataReader reader = this.ExecuteCommand("select name,user_type_id,system_type_id from sys.types");
            while (reader.Read())
            {
                DBType dbObject = new DBType(dbDatabase, (int)reader["user_type_id"], reader["name"] as string);
                dbDatabase.Types.Add(dbObject.Id, dbObject);
                this.typeCount++;

                TraceObject(dbObject);
            }

            reader.Close();
        }

        private void LoadTables(DBDatabase dbDatabase)
        {
            Trace.TraceInformation("------ Loading tables ------");

            SqlDataReader reader = this.ExecuteCommand("select name,object_id,schema_id from sys.tables");
            while (reader.Read())
            {
                DBTable dbObject = new DBTable(
                    dbDatabase
                    , dbDatabase.Schemas[(int)reader["schema_id"]]
                    , (int)reader["object_id"]
                    , reader["name"] as string);

                dbDatabase.Tables.Add(dbObject.Id, dbObject);
                this.tableCount++;

                TraceObject(dbObject);
            }

            reader.Close();
        }

        private void LoadTableColumns(DBDatabase dbDatabase)
        {
            Trace.TraceInformation("------ Loading table columns ------");

            SqlDataReader reader = this.ExecuteCommand("select * from sys.columns order by object_id, column_id");
            while (reader.Read())
            {
                int tableId = (int)reader["object_id"];

                if (dbDatabase.Tables.ContainsKey(tableId))
                {
                    DBTable table = dbDatabase.Tables[tableId];

                    DBColumn dbObject = new DBColumn(dbDatabase, (int)reader["column_id"], reader["name"] as string)
                    {
                        UserType = dbDatabase.Types[(int)reader["user_type_id"]],
                        SystemType = dbDatabase.Types[(byte)reader["system_type_id"]],
                        IsComputed = (bool)reader["is_computed"],
                        IsIdentity = (bool)reader["is_identity"],
                        IsNullable = (bool)reader["is_nullable"],
                        MaxLength = (short)reader["max_length"],
                        Precision = (byte)reader["precision"],
                        Scale = (byte)reader["scale"]
                    };

                    table.Columns.Add(dbObject.Id, dbObject);

                    Trace.TraceInformation("{2}.{1}", dbObject.Id, dbObject.Name, table.Name);

                    this.columnCount++;
                }
            }

            reader.Close();
        }

        private void LoadPrimaryKeys(DBDatabase dbDatabase)
        {
            Trace.TraceInformation("------ Loading primary keys ------");

            SqlDataReader reader = this.ExecuteCommand("select ic.* from sys.index_columns ic join sys.indexes i on ic.object_id = i.object_id and ic.index_id = i.index_id where is_primary_key = 1 order by ic.object_id, ic.index_id, ic.index_column_id");
            while (reader.Read())
            {
                int tableId = (int)reader["object_id"];

                if (dbDatabase.Tables.ContainsKey(tableId))
                {
                    DBTable table = dbDatabase.Tables[tableId];
                    DBColumn column = table.Columns[(int)reader["column_id"]];
                    table.PrimaryKey.Add((int)reader["index_column_id"], column);
                    column.IsPrimaryKey = true;

                    this.primaryKeyCount++;
                    Trace.TraceInformation("{0}.{1}", table.Name, column.Name);
                }
            }

            reader.Close();
        }

        private void LoadRelationships(DBDatabase dbDatabase)
        {
            Trace.TraceInformation("------ Loading relationships ------");

            SqlDataReader reader = this.ExecuteCommand("select * from sys.foreign_keys");
            while (reader.Read())
            {
                DBTable parentTable = dbDatabase.Tables[(int)reader["parent_object_id"]];
                DBTable referencedTable = dbDatabase.Tables[ (int)reader["referenced_object_id"]];

                DBRelationship dbObject = new DBRelationship(dbDatabase
                    , (int)reader["object_id"]
                    , reader["name"] as string
                    , parentTable
                    , referencedTable
                    );

                dbDatabase.Relationships.Add(dbObject.Id, dbObject);

                this.relationshipCount++;

                TraceObject(dbObject);
            }

            reader.Close();
        }

        private void LoadRelationshipColumns(DBDatabase dbDatabase)
        {
            Trace.TraceInformation("------ Loading relationship columns ------");

            SqlDataReader reader = this.ExecuteCommand("select * from sys.foreign_key_columns order by constraint_object_id,constraint_column_id");
            while (reader.Read())
            {
                DBTable parentTable = dbDatabase.Tables[(int)reader["parent_object_id"]];
                DBTable referencedTable = dbDatabase.Tables[(int)reader["referenced_object_id"]];

                DBRelationship relationship = dbDatabase.Relationships[(int)reader["constraint_object_id"]];

                DBColumn pcolumn = parentTable.Columns[(int)reader["parent_column_id"]];
                DBColumn rcolumn = referencedTable.Columns[(int)reader["referenced_column_id"]];

                pcolumn.IsForeignKey = true;

                DBRelationshipColumn dbObject = new DBRelationshipColumn(dbDatabase, (int)reader["constraint_column_id"], pcolumn, rcolumn);
                relationship.Columns.Add(dbObject.Id, dbObject);

                Trace.TraceInformation("{0}: {1}.{2} -> {3}.{4}", dbObject.Id, parentTable.Name, pcolumn.Name, referencedTable.Name, rcolumn.Name);

                this.relationshipColumnCount++;
            }

            reader.Close();
        }


        private void LoadProcedures(DBDatabase dbDatabase)
        {
            Trace.TraceInformation("------ Loading stored procedures ------");

            SqlDataReader reader = this.ExecuteCommand("select name,object_id,schema_id from sys.procedures");
            while (reader.Read())
            {
                DBProcedure dbObject = new DBProcedure(
                    dbDatabase
                    , dbDatabase.Schemas[(int)reader["schema_id"]]
                    , (int)reader["object_id"]
                    , reader["name"] as string);

                dbDatabase.Procedures.Add(dbObject.Id, dbObject);
                this.procedureCount++;

                TraceObject(dbObject);
            }

            reader.Close();
        }


        private void LoadParameters(DBDatabase dbDatabase)
        {
            Trace.TraceInformation("------ Loading paramters ------");

            SqlDataReader reader = this.ExecuteCommand("select * from sys.parameters order by object_id, parameter_id");
            while (reader.Read())
            {
                int tableId = (int)reader["object_id"];

                if (dbDatabase.Procedures.ContainsKey(tableId))
                {
                    DBProcedure procedure = dbDatabase.Procedures[tableId];

                    DBParameter dbObject = new DBParameter(dbDatabase, (int)reader["parameter_id"], (reader["name"] as string).Substring(1))
                    {
                        UserType = dbDatabase.Types[(int)reader["user_type_id"]],
                        SystemType = dbDatabase.Types[(byte)reader["system_type_id"]],
                        MaxLength = (short)reader["max_length"],
                        Precision = (byte)reader["precision"],
                        Scale = (byte)reader["scale"],
                        IsOutput = (bool)reader["is_output"]
                    };

                    procedure.Parameters.Add(dbObject.Id, dbObject);

                    Trace.TraceInformation("{2}.{1}", dbObject.Id, dbObject.Name, procedure.Name);

                    this.parameterCount++;
                }
            }

            reader.Close();
        }

        private static void TraceObject(DBObject dbObject)
        {
            Trace.TraceInformation("{0}: {1}", dbObject.Id, dbObject.Name);
        }

        private SqlDataReader ExecuteCommand(string sql)
        {
            SqlCommand command = this.connection.CreateCommand();
            command.CommandText = sql;
            command.CommandType = CommandType.Text;
            return command.ExecuteReader();
        }
    }
}
