﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Dax;
using System.Xml.Serialization;
using System.IO;

namespace Editor
{
    public partial class MainForm : Form
    {
        public string FileName
        {
            get
            {
                return this.fileName;
            }
            set
            {
                this.fileName = value;
                if (!string.IsNullOrEmpty(this.fileName))
                {
                    this.Text = Path.GetFileName(this.fileName) + " - Dax Generation Profile";
                }
                else
                {
                    this.Text = "New - Dax Generation Profile";
                }
            }
        }
        string fileName;

        public MainForm()
        {
            this.Font = SystemFonts.MessageBoxFont;

            InitializeComponent();

            // Load saved settings
            this.Location = UserSettings.Default.Location;
            this.Size = UserSettings.Default.Size;
            this.WindowState = UserSettings.Default.State;

            string[] args = Environment.GetCommandLineArgs();
            if (args.Length == 2)
            {
                OpenFile(args[1]);
            }
            else
            {
                NewFile();
            }
        }

        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (WindowState != FormWindowState.Minimized)
            {
                UserSettings.Default.State = WindowState;
            }
            else
            {
                UserSettings.Default.State = FormWindowState.Normal;
            }
        }

        private void MainForm_SizeChanged(object sender, EventArgs e)
        {
            if (WindowState == FormWindowState.Normal)
            {
                UserSettings.Default.Size = Size;
            }
        }

        private void MainForm_LocationChanged(object sender, EventArgs e)
        {
            if (WindowState == FormWindowState.Normal)
            {
                UserSettings.Default.Location = Location;
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.NewFile();
        }

        private void NewFile()
        {
            if (!string.IsNullOrEmpty(this.fileName) && !SaveDirty())
            {
                return;
            }

            GenerationProfile profile = new GenerationProfile();
            profile.ConnectionString = @"Data Source=server\sql;Initial Catalog=databse;Integrated Security=True";
            profile.DataAccess.BuildersFolder = "Builders";
            profile.DataAccess.RecordsFolder = "Records";
            profile.DataAccess.RepositoriesFolder = "Repositories";
            profile.DataAccess.ConcurrencyColumn = "Updated";
            profile.DataAccess.Namespace = "Dax.DataAccess";
            profile.StoredProcedures.Prefix = "gsp_";
            profile.StoredProcedures.UserPrefix = "usp_";
            profile.ORM.Enabled = true;
            profile.ORM.ObjectsFolder = "Objects";
            profile.ORM.RepositoriesFolder = "Repositories";
            profile.ORM.Namespace = "DataAccess.ORM";
            profile.DataAccessInterface.Namespace = "DataAccess.Interfaces";
            profile.IsDirty = false;

            ShowProfile(profile);
            this.FileName = "New Profile";
        }

        void ShowProfile(GenerationProfile profile)
        {
            this.profileControl.SetProfile(profile);
            this.profileControl.Show();
            this.saveAsMenuItem.Enabled = true;
            this.saveMenuItem.Enabled = true;
            this.saveToolButton.Enabled = true;
        }

        bool SaveDirty()
        {
            if (string.IsNullOrEmpty(this.fileName) || this.profileControl.profile.IsDirty)
            {
                var result = MessageBox.Show("Save changes to file?", "File Changed", MessageBoxButtons.YesNoCancel);

                switch (result)
                {
                    case DialogResult.Cancel:
                        return false;
                    case DialogResult.No:
                        return true;
                    case DialogResult.Yes:
                        return Save(this.fileName);
                    default:
                        break;
                }
            }

            return true;
        }

        bool SaveAs()
        {
            using (SaveFileDialog sfd = new SaveFileDialog())
            {
                if (sfd.ShowDialog(this) != System.Windows.Forms.DialogResult.OK)
                {
                    return false;
                }
                else
                {
                    this.Save(sfd.FileName);
                    return true;
                }
            }
        }

        bool Save(string fileName)
        {
            if (string.IsNullOrEmpty(fileName))
            {
                return SaveAs();
            }

            using (StreamWriter writer = new StreamWriter(fileName))
            {
                XmlSerializer serializer = new XmlSerializer(typeof(GenerationProfile));
                serializer.Serialize(writer, this.profileControl.profile);
                writer.Close();
                this.FileName = fileName;
                this.profileControl.profile.IsDirty = false;
            }
            return true;
        }

        private void openMenuItem_Click(object sender, EventArgs e)
        {
            this.OpenFile();
        }

        void OpenFile()
        {
            if (!SaveDirty())
            {
                return;
            }

            using (OpenFileDialog ofd = new OpenFileDialog())
            {
                if (ofd.ShowDialog(this) == System.Windows.Forms.DialogResult.OK)
                {
                    OpenFile(ofd.FileName);
                }
            }
        }

        void OpenFile(string fileName)
        {
            using (StreamReader reader = new StreamReader(fileName))
            {
                XmlSerializer serializer = new XmlSerializer(typeof(GenerationProfile));
                GenerationProfile profile = serializer.Deserialize(reader) as GenerationProfile;
                reader.Close();
                profile.IsDirty = false;
                ShowProfile(profile);
                this.FileName = fileName;
            }
        }

        private void saveMenuItem_Click(object sender, EventArgs e)
        {
            Save(this.fileName);
        }

        private void saveAsMenuItem_Click(object sender, EventArgs e)
        {
            SaveAs();
        }

        private void newToolButton_Click(object sender, EventArgs e)
        {
            this.NewFile();
        }

        private void openToolButton_Click(object sender, EventArgs e)
        {
            this.OpenFile();
        }

        private void saveToolButton_Click(object sender, EventArgs e)
        {
            Save(this.fileName);
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            using (AboutBox ab = new AboutBox())
            {
                ab.ShowDialog(this);
            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = !this.SaveDirty();

        }
    }
}
