﻿//-----------------------------------------------------------------------
// <copyright file="DBTableExtensions.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax.Generator
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Dax.Database;

    /// <summary>
    /// TODO: Describe the class
    /// </summary>
    public static class DBTableExtensions
    {
        public static GenerationProfile Profile { get; set; }

        public static string ColumnList(this DBTable table)
        {
            StringBuilder columnList = new StringBuilder();
            foreach (var column in table.Columns.Values)
            {
                columnList.AppendCommaList(column.Name);
            }
            return columnList.ToString();
        }

        public static string GetStoredProcedureName(this DBTable table)
        {
            return "[" + table.Schema.Name + "].[" + Profile.StoredProcedures.Prefix + table.Name + ".Get]";
        }

        public static string GetAllStoredProcedureName(this DBTable table)
        {
            return "[" + table.Schema.Name + "].[" + Profile.StoredProcedures.Prefix + table.Name + ".GetAll]";
        }

        public static string UpdateStoredProcedureName(this DBTable table)
        {
            return "[" + table.Schema.Name + "].[" + Profile.StoredProcedures.Prefix + table.Name + ".Update]";
        }

        public static string InsertStoredProcedureName(this DBTable table)
        {
            return "[" + table.Schema.Name + "].[" + Profile.StoredProcedures.Prefix + table.Name + ".Insert]";
        }

        public static string DeleteStoredProcedureName(this DBTable table)
        {
            return "[" + table.Schema.Name + "].[" + Profile.StoredProcedures.Prefix + table.Name + ".Delete]";
        }

        public static IEnumerable<DBColumn> UpdatableColumns(this DBTable table)
        {
            return table.Columns.Values.Where(c => !c.IsComputed && !c.IsIdentity && !c.IsConcurrencyColumn()).ToList();
        }

        public static IEnumerable<DBColumn> DatabaseUpdatedColumns(this DBTable table)
        {
            return table.Columns.Values.Where(c => c.IsComputed || c.IsIdentity || c.IsConcurrencyColumn()).ToList();
        }

        public static DBColumn ConcurrencyColumn(this DBTable table)
        {
            return (from c in table.Columns.Values
                    where c.IsConcurrencyColumn()
                    select c).FirstOrDefault();
        }

        public static bool HasNonIdentityPrimaryKeys(this DBTable table)
        {
            if (table.PrimaryKey.Values.Where(x => !x.IsIdentity).ToList().Count > 0)
            {
                return true;
            }

            return false;
        }

        public static IEnumerable<DBProcedure> UserProcedures(this DBTable table)
        {
            string uspName = Profile.StoredProcedures.UserPrefix + table.Name;

            var list = from t in table.DBDatabase.Procedures.Values.Where(p => p.Name.StartsWith(uspName))
                       select t;

            return list;
        }

        #region Data Access
        public static string DARecordClassName(this DBTable table)
        {
            return table.Name.Singlular() + "Record";
        }

        public static string DABuilderClassName(this DBTable table)
        {
            return table.Name.Singlular() + "Builder";
        }

        public static string DARepositoryClassName(this DBTable table)
        {
            return table.Name.Singlular() + "DataRepository";
        }
        public static string DARepositoryPrivateVariableName(this DBTable table)
        {
            return table.Name.ToCamel() + "DataRepository";
        }

        public static string DARepositoryPublicVariableName(this DBTable table)
        {
            return table.Name.Plural();
        }

        public static string DAReferencedName(this DBTable table)
        {
            return table.Name.Singlular();
        }
        #endregion

        #region Data Access SQL

        public static string PrimaryKeyParameterList(this DBTable table, bool withType, bool withIdentity)
        {
            StringBuilder s = new StringBuilder();

            IEnumerable<DBColumn> columns;
            if (withIdentity)
            {
                columns = table.PrimaryKey.Values;
            }
            else
            {
                columns = table.PrimaryKey.Values.Where(x => !x.IsIdentity).ToList();
            }

            foreach (var column in columns)
            {
                if (withType)
                {
                    s.AppendCommaList(string.Format("{0} {1}", column.CSharpType(), column.Name.ToCamel()));
                }
                else
                {
                    s.AppendCommaList(column.Name.ToCamel());
                }
            }
            return s.ToString();
        }

        public static string PrimaryKeyList(this DBTable table, string qualifier)
        {
            StringBuilder s = new StringBuilder();

            foreach (var column in table.PrimaryKey.Values)
            {
                s.AppendCommaList(string.Format("{0}.{1}", qualifier, column.Name));
            }
            return s.ToString();
        }

        #endregion

        #region ORM
        public static string ORMObjectClassName(this DBTable table)
        {
            return "ORM" + table.Name.Singlular();
        }

        public static string ORMRepositoryClassName(this DBTable table)
        {
            return "ORM" + table.Name.Singlular() + "Repository";
        }

        public static string ORMRepositoryContextPropertyName(this DBTable table)
        {
            return table.Name.Plural();
        }

        public static string ORMRepositoryPrivateVariableName(this DBTable table)
        {
            return "m" + table.ORMRepositoryClassName();
        }

        public static string ORMObjectVariableName(this DBTable table)
        {
            return "orm" + table.Name.Singlular();
        }

        public static string ORMObjectPrivateVariableName(this DBTable table)
        {
            return "m" + table.ORMObjectClassName();
        }

        public static string ORMObjectKeyType(this DBTable table)
        {
            if (table.HasCompositePrimaryKey)
            {
                return table.ORMObjectClassName() + ".CompositeKey";
            }
            else
            {
                return table.PrimaryKey.Values.First().CSharpType();
            }
        }

        public static string ORMObjectKeyName(this DBTable table)
        {
            if (table.HasCompositePrimaryKey)
            {
                return "Key";
            }
            else
            {
                return table.PrimaryKey.Values.First().Name;
            }
        }

        public static IEnumerable<DBColumn> ORMObjectPropertyColumns(this DBTable table)
        {
            var columns = from c in table.Columns.Values
                          where !(c.IsForeignKey & !c.IsPrimaryKey)
                          select c;

            return columns;
        }

        #endregion
    }
}
