﻿//-----------------------------------------------------------------------
// <copyright file="Generator.cs" company="Russell Wilkins">
//     Copyright (c) Russell Wilkins 2011. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------
namespace Dax.Generator
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Dax.Database;
    using System.Diagnostics;
    using System.IO;
    using System.Collections.Concurrent;
    using System.Threading.Tasks;

    /// <summary>
    /// The base generator class for all code generators.
    /// </summary>
    public abstract class Generator
    {
        private GenerationProfile profile;
        private DBDatabase database;
        private string name;
        private string projectFile;
        private ConcurrentBag<GenerationFile> generationFiles = new ConcurrentBag<GenerationFile>();
        private IDictionary<string, object> session = new ConcurrentDictionary<string, object>();
        private bool multithread;

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the Generator class.
        /// </summary>
        public Generator(GenerationProfile profile, DBDatabase database, string projectFile, string name, bool multithread)
            : this(profile, database, name, multithread)
        {
            this.projectFile = projectFile;
            this.session.Add("profile", profile);
            this.session.Add("database", database);
            this.Session.Add("DataAccessInterfaceNamespace", this.Profile.DataAccessInterface.Namespace);
            this.Session.Add("DataAccessNamespace", this.Profile.DataAccess.Namespace);
        }

        public Generator(GenerationProfile profile, DBDatabase database, string name, bool multithread)
        {
            this.profile = profile;
            this.database = database;
            this.name = name;
            this.multithread = multithread;
        }
        #endregion

        /// <summary>
        /// Gets the summary.
        /// </summary>
        /// <value>The summary.</value>
        public GenerationProfile Profile
        {
            get
            {
                return this.profile;
            }
        }

        /// <summary>
        /// Gets the summary.
        /// </summary>
        /// <value>The summary.</value>
        public DBDatabase Database
        {
            get
            {
                return this.database;
            }
        }

        public ConcurrentBag<GenerationFile> GenerationFiles
        {
            get
            {
                return this.generationFiles;
            }
        }


        public IDictionary<string, object> Session
        {
            get
            {
                return this.session;
            }
        }

        public void Generate()
        {
            Trace.TraceInformation("------ {0} code generation started ------", this.name);

            this.GenerationStart();

            this.GenerateDatabase(this.database);

            // Tables
            if (this.multithread)
            {
                Parallel.ForEach(this.database.Tables.Values, table =>
                    {
                        this.GenerateTable(table);
                    });
            }
            else
            {
                foreach (var table in this.database.Tables.Values)
                {
                    this.GenerateTable(table);
                }
            }

            this.GenerationEnd();

            if (this.generationFiles.Count > 0)
            {
                FileGenerator fileGenerator = new FileGenerator(this.Profile, this.session);
                fileGenerator.Generate(this.generationFiles);

                if (!string.IsNullOrEmpty(this.projectFile))
                {
                    this.UpdateProject(this.generationFiles, this.projectFile);
                }
            }

            Trace.TraceInformation("========== {0} generation: {1} files generated ==========", this.name, this.generationFiles.Count);
        }

        protected virtual void GenerateDatabase(DBDatabase database) { }
        protected virtual void GenerateTable(DBTable table) { }
        protected virtual void GenerationStart() { }
        protected virtual void GenerationEnd() { }

        protected void UpdateProject(ConcurrentBag<GenerationFile> files, string projectFile)
        {
            if (string.IsNullOrEmpty(projectFile) || !File.Exists(projectFile))
            {
                return;
            }

            Trace.TraceInformation("Updating project file " + projectFile);
            VSProject p = new VSProject(projectFile);
            foreach (var item in files)
            {
                p.AddCompileItem(item.FileName);
            }
            p.Save();
        }
    }
}
