﻿//----------------------------------------------------------------------------
// <copyright file="SqlSchemaBuilder.cs" company="Russell Wilkins">
//     Copyright (c) 2011 Russell Wilkins 2011. All rights reserved.
//
//     The use of this software is governed by the Microsoft Public License
//     which is included with this distribution.
// </copyright>
//----------------------------------------------------------------------------
namespace Dax.Schema.SqlClient
{
    using System;
    using System.Collections.Generic;
    using System.Data;
    using System.Data.SqlClient;
    using System.Diagnostics;
    using System.Linq;
    using System.Text;
    using Dax.Schema;

    /// <summary>
    /// Class to build DatabaseSchema objects from an SqlServer connection.
    /// </summary>
    public class SqlSchemaBuilder
    {
        private SqlConnection connection;
        private DatabaseSchema schema;
        private int tableCount;
        private int typeCount;
        private int columnCount;
        private int indexCount;
        private int relationshipCount;
        private int relationshipColumnCount;
        private int procedureCount;
        private string connectionString;

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the SqlSchemaBuilder class.
        /// </summary>
        /// <param name="connection">The database connection.</param>
        public SqlSchemaBuilder(string connectionString)
        {
            this.connectionString = connectionString;
        }
        #endregion

        /// <summary>
        /// Builds and returns a DatabaseSchema for all build options.
        /// </summary>
        /// <returns>The DatabaseSchema</returns>
        public DatabaseSchema GetDatabaseSchema()
        {
            return GetDatabaseSchema(SchemaBuildOptions.Procedures | SchemaBuildOptions.Tables);
        }

        /// <summary>
        /// Builds and returns a DatabaseSchema based on the passed options.
        /// </summary>
        /// <param name="options">The build options for the returned DatabaseSchema.</param>
        /// <returns>The DatabaseSchema</returns>
        public DatabaseSchema GetDatabaseSchema(SchemaBuildOptions options)
        {
            Trace.WriteLine(string.Format("------ Schema build started: {0} ------", connectionString));

            using (SqlConnection connection = new SqlConnection(this.connectionString))
            {
                connection.Open();
                this.connection = connection;

                this.schema = new DatabaseSchema(connection.Database);

                this.LoadTypes();

                if ((options & SchemaBuildOptions.Tables) == SchemaBuildOptions.Tables)
                {
                    this.LoadTables();
                    this.LoadColumns();
                    this.LoadPrimaryKeys();
                    this.LoadForeignKeys();
                    this.LoadForeignKeyColumns();
                }

                if ((options & SchemaBuildOptions.Procedures) == SchemaBuildOptions.Procedures)
                {
                    this.LoadStoredProcedures();
                }

                connection.Close();
            }

            Trace.WriteLine(string.Format(
                "========== Schema build: {0} types, {1} tables, {2} indexes, {3} columns, {4} procedures, {5} relationships ==========",
                this.typeCount,
                this.tableCount,
                this.indexCount,
                this.columnCount,
                this.procedureCount,
                this.relationshipCount));

            return this.schema;
        }

        private SqlDataReader ExecuteCommand(string sql)
        {
            SqlCommand command = this.connection.CreateCommand();
            command.CommandText = sql;
            command.CommandType = CommandType.Text;
            return command.ExecuteReader();
        }

        private void LoadTypes()
        {
            Trace.WriteLine("------ Building types ------");
            Trace.Indent();

            SqlDataReader reader = this.ExecuteCommand("select name,user_type_id,system_type_id from sys.types");
            while (reader.Read())
            {
                Trace.WriteLine(string.Format("Creating {0}...", reader["name"] as string));
                DatabaseType type = new DatabaseType(this.schema, reader["name"] as string, (int)reader["user_type_id"], (byte)reader["system_type_id"]);
                this.schema.AddType(type);
                this.typeCount++;
            }

            reader.Close();

            Trace.Unindent();
        }

        private void LoadStoredProcedures()
        {
            Trace.WriteLine("------ Building  procedures ------");
            Trace.Indent();

            SqlDataReader reader = this.ExecuteCommand("select name,object_id from sys.procedures where type = 'P'");
            while (reader.Read())
            {
                Trace.WriteLine((reader["name"] as string) + ": Creating procedure");
                DatabaseProcedure procedure = new DatabaseProcedure(reader["name"] as string, (int)reader["object_id"]);
                this.schema.AddProcedure(procedure);
                this.procedureCount++;
            }

            reader.Close();

            Trace.Unindent();
        }

        private void LoadTables()
        {
            Trace.WriteLine("------ Building tables ------");
            Trace.Indent();

            SqlDataReader reader = this.ExecuteCommand("select name,object_id from sys.tables");
            while (reader.Read())
            {
                Trace.WriteLine((reader["name"] as string) + ": Creating table");
                DatabaseTable table = new DatabaseTable(reader["name"] as string, (int)reader["object_id"]);
                this.schema.AddTable(table);
                this.tableCount++;
            }

            reader.Close();
            Trace.Unindent();
        }

        private void LoadColumns()
        {
            Trace.WriteLine("------ Building columns ------");
            Trace.Indent();

            SqlDataReader reader = this.ExecuteCommand("select * from sys.columns");
            while (reader.Read())
            {
                int objectId = (int)reader["object_id"];

                if (this.schema.ContainsTable(objectId))
                {
                    DatabaseTable table = this.schema.GetTableByObjectId(objectId);

                    Trace.WriteLine(table.Name + ": Adding column " + reader["name"] as string);

                    DatabaseColumn column = new DatabaseColumn(reader["name"] as string);
                    column.IsNullable = (bool)reader["is_nullable"];
                    column.Id = (int)reader["column_id"];
                    object o = reader["max_length"];
                    column.MaxLength = (short)reader["max_length"];
                    column.Precision = (byte)reader["precision"];
                    column.Scale = (byte)reader["scale"];
                    column.IsIdentity = (bool)reader["is_identity"];
                    column.IsComputed = (bool)reader["is_computed"];
                    column.DatabaseType = this.schema.GetType((int)reader["user_type_id"]);

                    table.AddColumn(column);
                    this.columnCount++;
                }
            }

            reader.Close();
            Trace.Unindent();
        }

        private void LoadPrimaryKeys()
        {
            Trace.WriteLine("------ Building primary keys ------");
            Trace.Indent();

            SqlDataReader reader = this.ExecuteCommand("select ic.* from sys.index_columns ic join sys.indexes i on ic.object_id = i.object_id and ic.index_id = i.index_id where is_primary_key = 1 order by ic.object_id, ic.index_id, ic.index_column_id");
            while (reader.Read())
            {
                int objectId = (int)reader["object_id"];

                if (this.schema.ContainsTable(objectId))
                {
                    DatabaseTable table = this.schema.GetTableByObjectId(objectId);
                    DatabaseColumn column = table.GetColumnById((int)reader["column_id"]);

                    Trace.WriteLine(table.Name + ": Adding primary key " + column.Name);
                    table.AddPrimaryKeyColumn((int)reader["index_column_id"], (int)reader["column_id"]);
                    this.indexCount++;
                }
            }

            reader.Close();
            Trace.Unindent();
        }

        private void LoadForeignKeys()
        {
            Trace.WriteLine("------ Building foreign key relationships ------");
            Trace.Indent();

            SqlDataReader reader = this.ExecuteCommand("select * from sys.foreign_keys");
            while (reader.Read())
            {
                int parentObjectId = (int)reader["parent_object_id"];
                int referencedObjectId = (int)reader["referenced_object_id"];

                DatabaseTable parentTable = this.schema.GetTableByObjectId(parentObjectId);
                DatabaseTable referencedTable = this.schema.GetTableByObjectId(referencedObjectId);

                DatabaseRelationship relationship = new DatabaseRelationship((string)reader["name"], (int)reader["object_id"], parentTable, referencedTable);

                Trace.WriteLine(parentTable.Name + ": Adding foreign key relationship " + relationship.Name);

                schema.AddRelationship(relationship);

                parentTable.AddRelationship(relationship);
                if (parentTable != referencedTable)  
                {
                    referencedTable.AddRelationship(relationship);
                }

                this.relationshipCount++;
            }

            reader.Close();
            Trace.Unindent();
        }

        private void LoadForeignKeyColumns()
        {
            Trace.WriteLine("------ Building foreign key columns ------");
            Trace.Indent();

            SqlDataReader reader = this.ExecuteCommand("select * from sys.foreign_key_columns order by constraint_object_id,constraint_column_id");
            while (reader.Read())
            {
                int objectId = (int)reader["constraint_object_id"];
                int parentObjectId = (int)reader["parent_object_id"];

                DatabaseTable table = this.schema.GetTableByObjectId(parentObjectId);
                DatabaseRelationship relationship = table.GetRelationshipById(objectId);

                Trace.WriteLine(relationship.Name + ": Adding foreign key column" + relationship.Name);

                DatabaseColumn pc = relationship.ParentTable.GetColumnById((int)reader["parent_column_id"]);
                relationship.AddParentColumn((int)reader["constraint_column_id"], pc);

                DatabaseColumn rc = relationship.ReferencedTable.GetColumnById((int)reader["referenced_column_id"]);
                relationship.AddReferencedColumn((int)reader["constraint_column_id"], rc);

                pc.IsForeignKey = true;
                this.relationshipColumnCount++;
            }

            reader.Close();
            Trace.Unindent();
        }
    }
}
